#ifndef	__C3D_H__
#include "C3D.h"
#endif

///////////////////////////////////////////////////////////////////////
//  C3D file reader and writer
///////////////////////////////////////////////////////////////////////


float ConvertDecToFloat(char bytes[4]) 
{
    char p[4]; 
    p[0] = bytes[2]; 
    p[1] = bytes[3]; 
    p[2] = bytes[0]; 
    p[3] = bytes[1]; 
    if (p[0] || p[1] || p[2] || p[3]) 
        --p[3];          // adjust exponent 
    return *(float*)p; 
} 
  

void ConvertFloatToDec(float f, char* bytes) 
{
    char* p = (char*)&f; 
    bytes[0] = p[2]; 
    bytes[1] = p[3]; 
    bytes[2] = p[0]; 
    bytes[3] = p[1]; 
    if (bytes[0] || bytes[1] || bytes[2] || bytes[3]) 
        ++bytes[1];      // adjust exponent 
}


bool LoadC3DFile( const char *fileName, std::vector<Vec3d> & mPointData, int *nFrame, int *nMarker )
{
	char buf[C3D_REC_SIZE];
    FILE *file;
	int i, j;
	Vec3d v;
	c3d_head hdr;
	c3d_param param;
	c3d_frameSI frameSI;
	c3d_frame frame;
	bool bDecFmt = true;

	if ( (file=fopen( fileName, "rb" )) == NULL )  
		return false; 
	 
	//get the header
	if (!fread(buf, C3D_REC_SIZE, 1, file))
		return false;
	memcpy(&hdr, buf, sizeof(hdr));

		//get number format
	if (hdr.rec_start > 2)
	{
		if (!fread(buf, C3D_REC_SIZE, 1, file))
			return false;
		memcpy(&param, buf, sizeof(param));
		if (param.ftype == 84)
			bDecFmt = false;
	}

	//convert if in dec format
	if (bDecFmt)
	{
		hdr.freq = ConvertDecToFloat((char*)&hdr.freq);
		hdr.scale = ConvertDecToFloat((char*)&hdr.scale);
	}

	int mNumFrames = hdr.end_frame - hdr.start_frame + 1;
	int mNumMarkers = hdr.pnt_cnt;
	double mC3DScale = hdr.scale;

	*nMarker = mNumMarkers;
	*nFrame = mNumFrames;

	float pntScale = (hdr.scale < 0) ? 1 : hdr.scale;

	//eat parameter records
	for (i = 3; i < hdr.rec_start; i++)
		if (!fread(buf, C3D_REC_SIZE, 1, file))
			return false;
	
	// start retrieving data
	mPointData.resize(mNumFrames*mNumMarkers);

	int iRecSize;
	if (mC3DScale < 0)
		iRecSize = sizeof(c3d_frame) + ( hdr.a_channels * hdr.a_frames * sizeof(float));
	else
		iRecSize = sizeof(c3d_frameSI) + ( hdr.a_channels * hdr.a_frames * sizeof(short));

	for( i=0;i<mNumFrames;i++ )
		for( j=0;j<mNumMarkers;j++ )
		{
			if (!fread(buf, iRecSize, 1, file))
				return false;
			if (mC3DScale < 0)
			{
				memcpy(&frame, buf, sizeof(frame));
				v[0] = frame.y / 1000.0;
				v[1] = frame.z / 1000.0;
				v[2] = frame.x / 1000.0;
			}
			else
			{
				memcpy(&frameSI, buf, sizeof(frameSI));
				v[0] = (float)frameSI.y * pntScale / 1000.0;
				v[1] = (float)frameSI.z * pntScale / 1000.0;
				v[2] = (float)frameSI.x * pntScale / 1000.0;
			}
			mPointData[i * mNumMarkers + j] = v;
		}

	fclose(file);

	const char *pch = strrchr(fileName, '\\');  //clip leading path //LUIS :const
	if (pch) fileName = pch+1;

	return true;
}