// The sample robotarm model.  You should build a file
// very similar to this for when you make your model.

#pragma warning(disable : 4018)
#pragma warning(disable : 4267)
#pragma warning(disable : 4311)
#pragma warning(disable : 4312)
#pragma warning(disable : 4244)
#pragma warning(disable : 4305)

#include "modelerview.h"
#include "modelerapp.h"
#include "modelerdraw.h"
#include "particleSystem.h"


#include <FL/gl.h>
#include <FL/glut.H>
#include <cstdlib>

//using namespace std;

#define M_DEFAULT 2.0f
#define M_OFFSET 3.0f
#define P_OFFSET 0.3f
#define MAX_VEL 200
#define MIN_STEP 0.1

#define RIGHT_HAND 1
#define LEFT_HAND 0

class StackMatrix;

class StackMatrix {
    public:
    Mat4f mat;
    StackMatrix *under;
    StackMatrix(StackMatrix *);
};

StackMatrix::StackMatrix(StackMatrix *u) {
    under = u;
    if(under == NULL) {
        mat = vl_1;
    } else {
        mat = Mat4f(u->mat);
    }
}


Vec4f getEmitterWC(void);
Mat4f getModelViewMatrix(void);
Vec4f emitterPos;
Vec3f emitterNormal;
Mat4f worldMat;
Mat4f worldTransform;
StackMatrix *matStack;

// This is a list of the controls for the RobotArm
// We'll use these constants to access the values 
// of the controls from the user interface.
enum RobotArmControls
{ 
    BASE_ROTATION=0,DISPLACEMENT, LOWER_TILT_LEFT, LOWER_TILT_RIGHT, /*UPPER_TILT,*/ CLAW_ROTATION,
    NECK_LENGTH, UPPER_LENGTH_L, LOWER_LENGTH_L, UPPER_LENGTH_R, LOWER_LENGTH_R, HAND_ANGLE_L,
    HAND_ANGLE_R, COMPOUND_L, COMPOUND_R, WALL1, WALL2, WALL3,

    NUMCONTROLS, 
};

void ground(float h);
void wheels(float h);
void base(float h);
void rotation_base(float h);
void neck(float h);
void head(float l);
void upper_arm(float h);
void lower_arm(float h);
void claw(float h);
void y_box(float h);
void hand(float scale, int side);



// To make a RobotArm, we inherit off of ModelerView
class RobotWallE : public ModelerView 
{
public:
    RobotWallE(int x, int y, int w, int h, char *label) 
        : ModelerView(x,y,w,h,label) {}
    virtual void draw();
};

// We need to make a creator function, mostly because of
// nasty API stuff that we'd rather stay away from.
ModelerView* createRobotWallE(int x, int y, int w, int h, char *label)
{ 
    return new RobotWallE(x,y,w,h,label); 
}

// We'll be getting the instance of the application a lot; 
// might as well have it as a macro.
#define VAL(x) (ModelerApplication::Instance()->GetControlValue(x))


void rotatef(float theta, float x, float y, float z) {
    Mat4f newTransform;
    glRotatef(theta, x, y, z);
    newTransform = xform(matStack->mat, HRot4f(Vec3f(x, y, z), theta * (vl_pi / 180)));
    
    for(int i = 0; i < 4; i++)
        for(int j = 0; j < 4; j++)
            if(fabs(newTransform[i][j]) < 0.0001)
                newTransform[i][j] = 0;
            
    matStack->mat = newTransform;
    //std::cout << "World transform rotated by " << theta << " around " << Vec3f(x, y, z) << ":\n" << worldTransform << endl;
}

void translatef(float x, float y, float z) {
    Mat4f newTransform;
    glTranslatef(x, y, z);
    newTransform = xform(matStack->mat, HTrans4f(Vec3f(x, y, z)));
    matStack->mat = newTransform;
    //std::cout << "World transform translated by " << Vec3f(x, y, z) << ":\n" << worldTransform << endl;

}

void scalef(float x, float y, float z) {
    Mat4f newTransform;
    glScalef(x, y, z);
    newTransform = xform(matStack->mat, HScale4f(Vec3f(x, y, z)));
    matStack->mat = newTransform;
    //std::cout << "World transforms caled by " << Vec3f(x, y, z) << ":\n" << worldTransform << endl;
}

void pushMatrix() {
    glPushMatrix();
    StackMatrix *newM = new StackMatrix(matStack);
    matStack = newM;
}

void popMatrix() {
    StackMatrix *old = matStack;
    glPopMatrix();
    if(matStack->under != NULL) {
        matStack = matStack->under;
        delete old;
    }
}

Vec3f worldCoords(Vec4f objectCoords) {
    Vec4f result = xform(matStack->mat, objectCoords);
    return Vec3f(result[0], result[1], result[2]);
}

Vec3f worldCoords(Vec3f objectCoords) {
    return worldCoords(Vec4f(objectCoords, 1.0));
}

Vec3f worldCoords(float x, float y, float z) {
    return worldCoords(Vec3f(x, y, z));
}


// We are going to override (is that the right word?) the draw()
// method of ModelerView to draw out RobotArm
void RobotWallE::draw()
{
    /* pick up the slider values */

    float theta = VAL( BASE_ROTATION );
    float disp = VAL(DISPLACEMENT);
    float phi_l = VAL( LOWER_TILT_LEFT );
    float phi_r = VAL( LOWER_TILT_RIGHT );
    //float psi = VAL( UPPER_TILT );
    float cr = VAL( CLAW_ROTATION );
    //float h1 = VAL( BASE_LENGTH );
    float h1 = 2.0;
    float neckH = VAL(NECK_LENGTH);
    float h2_l = VAL( UPPER_LENGTH_L );
    float h3_l = VAL( LOWER_LENGTH_L );
    float h2_r = VAL( UPPER_LENGTH_R );
    float h3_r = VAL( LOWER_LENGTH_R );
    float hand_angle_l = VAL ( HAND_ANGLE_L );	//	from 0 to 1
    float hand_angle_r = VAL ( HAND_ANGLE_R );	//	from 0 to 1
    float compound_l = VAL ( COMPOUND_L );	//	from 0 to 1. rotate arm and open claw.  
    float compound_r = VAL (COMPOUND_R);  //	we'll add this to whatever the value is.

		
		hand_angle_l += compound_l*1.2;
		hand_angle_r += compound_r*1.2;
		phi_l += compound_l * 50;
		phi_r += compound_r * 50;
		

    // This call takes care of a lot of the nasty projection 
    // matrix stuff
    ModelerView::draw();
    ModelerApplication::Instance()->GetParticleSystem()->drawWalls();
    //ModelerApplication::Instance()->GetParticleSystem()->setWallCoeffs(VAL(WALL1), VAL(WALL2), VAL(WALL3));
    //worldTransform = vl_1;
    matStack = new StackMatrix(NULL);
    //std::cout << "World matrix initialized: " << worldTransform << std::endl;

    pushMatrix();
    static GLfloat lmodel_ambient[] = {0.4,0.4,0.4,1.0};

    // define the model


    ground(-0.2);
    
    rotatef( theta, 0.0, 1.0, 0.0 );
    translatef(disp, 0.0, 0.0);
    
    wheels(1.0);
    //base(0.8);

    translatef( 0.0, 1.0, 0.0 );			// move to the top of the base
    //rotatef( theta, 0.0, 1.0, 0.0 );		// turn the whole assembly around the y-axis. 
    rotation_base(h1);						// draw the rotation base

    ///// LEFT ARM /////
    pushMatrix();
    //Upper-Left ARM:
    translatef( 0.25, 1.0, 1.25 );
    rotatef( phi_l, 0.0, 0.0, 1.0 );		// rotate around the z-axis for the upper arm    
    upper_arm(h2_l);
    //Lower-Left ARM:
    translatef( 0.0, h2_l, 0.0 );			// move to the top of the lower arm
    //rotatef( psi, 0.0, 0.0, 1.0 );
    lower_arm(h3_l);
    translatef( 0.0, h3_l, 0.0);
    hand(hand_angle_l, LEFT_HAND);
    popMatrix();

    ///// RIGHT ARM /////
    pushMatrix();
    //Upper-Left ARM:
    translatef( 0.25, 1.00, -1.25 );
    rotatef( phi_r, 0.0, 0.0, 1.0 );		// rotate around the z-axis for the upper arm    
    upper_arm(h2_r);
    //Lower-Left ARM:
    translatef( 0.0, h2_r, 0.0 );			// move to the top of the lower arm
    //rotatef( psi, 0.0, 0.0, 1.0 );
    lower_arm(h3_r);
    
    //EMITTER:
    pushMatrix();
    translatef(-0.5, 0.5, -0.5);
    emitterPos = Vec4f(worldCoords(0.5, 1.0, 0.5), 1.0);
    emitterNormal = Vec3f(0.0, 0.0, 0.0);
    //std::cout << "EmitterPos: " << emitterPos << std::endl;
    ModelerApplication::Instance()->GetParticleSystem()->drawEmitter(&emitterNormal); //  sets worldNormal to local coords
    emitterNormal = norm(worldCoords(Vec4f(emitterNormal, 0.0)));
    //std::cout << "EmitterNorm: " << emitterNormal << std::endl;
    ModelerApplication::Instance()->GetParticleSystem()->setEmitterPosition(emitterPos[0], emitterPos[1], emitterPos[2], theta, phi_r);
    ModelerApplication::Instance()->GetParticleSystem()->setEmitterNormal(emitterNormal[0], emitterNormal[1], emitterNormal[2]);
    popMatrix();   
    
    translatef( 0.0, h3_r, 0.0);
    hand(hand_angle_r, RIGHT_HAND);
    
    popMatrix();
    
    translatef( 0.0, h1, 0.0 );			// move to the top of the base   
    neck(neckH);

    translatef(0.0, neckH, 0.0);          //Move to the top of the neck
    head(1.5);

    popMatrix();

    /*
    //draw line at emitter normal
    glPointSize(1.0);
    glTranslatef(emitterPos[0], emitterPos[1], emitterPos[2]);
    glBegin(GL_LINES);
    glVertex3f(0.0, 0.0, 0.0);
    glVertex3f(emitterNormal[0], emitterNormal[1], emitterNormal[2]);
    glEnd();
    */
    
    
    //*** DON'T FORGET TO PUT THIS IN YOUR OWN CODE **/
    endDraw();
    delete matStack;
}

Vec4f getEmitterWC() {
    //worldMat = getModelViewMatrix();
	//std::cout << "Matrix: " << worldTransform << std::endl;
    Vec4f pos(0.0, 0.0, 0.0, 1.0);
    return xform(worldTransform, pos);
}

Mat4f getModelViewMatrix()
{
    GLfloat m[16];
    glGetFloatv(GL_MODELVIEW_MATRIX, m);
    glGetFloatv(GL_MODELVIEW_MATRIX, m);
    Mat4f matMV(m[0], m[1], m[2], m[3],
        m[4], m[5], m[6], m[7],
        m[8], m[9], m[10], m[11],
        m[12], m[13], m[14], m[15] );    
    return matMV.Transpose();
    // because the matrix GL returns is column major
}

void ground(float h) 
{
    glDisable(GL_LIGHTING);
    glColor3f(0.65,0.45,0.2);
    glPushMatrix();
    glScalef(30,0,30);
    y_box(h);
    glPopMatrix();
    glEnable(GL_LIGHTING);
}

void wheels(float h) {
    setDiffuseColor( 0.25, 0.25, 0.25 );
    setAmbientColor( 0.25, 0.25, 0.25 );
    pushMatrix();
    pushMatrix();
    translatef(1.0, h / 2.0, 0.75);
    drawCylinder(0.5, h / 2.0, h / 2.0); //rear-left wheel
    popMatrix();
    pushMatrix();
    translatef(1.0, h / 2.0, -1.25);
    drawCylinder(0.5, h / 2.0, h / 2.0); //rear-right wheel
    popMatrix();
    pushMatrix();
    translatef(-1.0, h / 2.0, 0.75);
    drawCylinder(0.5, h / 2.0, h / 2.0); //front-left wheel
    popMatrix();
    pushMatrix();
    translatef(-1.0, h / 2.0, -1.25);
    drawCylinder(0.5, h / 2.0, h / 2.0); //front-right wheel
    popMatrix();
    scalef(4.0f, h, 4.0f);
    y_box(1.0f);
    popMatrix();
}
/*
void base(float h) {
    setDiffuseColor( 0.25, 0.25, 0.25 );
    setAmbientColor( 0.25, 0.25, 0.25 );
    pushMatrix();
    pushMatrix();
    translatef(1.0, h / 2.0, 0.75);
    drawCylinder(0.25, h / 2.0, h / 2.0);
    popMatrix();
    pushMatrix();
    translatef(1.0, h / 2.0, -1.0);
    drawCylinder(0.25, h / 2.0, h / 2.0);
    popMatrix();
    pushMatrix();
    translatef(-1.0, h / 2.0, 0.75);
    drawCylinder(0.25, h / 2.0, h / 2.0);
    popMatrix();
    pushMatrix();
    translatef(-1.0, h / 2.0, -1.0);
    drawCylinder(0.25, h / 2.0, h / 2.0);
    popMatrix();
    scalef(4.0f, h, 4.0f);
    y_box(1.0f);
    popMatrix();
}
*/
void rotation_base(float h) {
    setDiffuseColor( 0.85, 0.75, 0.25 );
    setAmbientColor( 0.95, 0.75, 0.25 );
    pushMatrix();
    pushMatrix();
    scalef(4.0, h, 4.0);
    y_box(1.0f); // the rotation base
    popMatrix();
    //setDiffuseColor( 0.15, 0.15, 0.65 );
    //setAmbientColor( 0.15, 0.15, 0.65 );
    //pushMatrix();
    //translatef(-0.5, h, -0.6);
    //scalef(2.0, h, 1.6);
    //y_box(1.0f); // the console
    //popMatrix();
    //setDiffuseColor( 0.65, 0.65, 0.65 );
    //setAmbientColor( 0.65, 0.65, 0.65 );
    //pushMatrix();
    //translatef( 0.5, h, 0.6 );
    //rotatef( -90.0, 1.0, 0.0, 0.0 );
    //drawCylinder( h, 0.05, 0.05 ); // the pipe
    //popMatrix();
    popMatrix();
}

void neck(float h) {
    setDiffuseColor( 0.65, 0.65, 0.65 );
    setAmbientColor( 0.65, 0.65, 0.65 );
    pushMatrix();
    rotatef(-90.0, 1.0, 0.0, 0.0);
    drawCylinder( h, 0.2, 0.2 );
    popMatrix();
}

void head(float l) {
    setDiffuseColor( 0.65, 0.65, 0.65 );
    setAmbientColor( 0.65, 0.65, 0.65 );
    pushMatrix();   
    drawSphere(0.4); //Neck-eye connection
    rotatef(-90.0, 0.0, 1.0, 0.0);
    pushMatrix();
    
    translatef(0.5, 0.0, -0.75);
    drawCylinder(l, 0.4, 0.4); //left eye

    pushMatrix();
    setDiffuseColor( 0.25, 0.65, 0.65 );
    setAmbientColor( 0.25, 0.65, 0.65 );
    translatef(0.0, 0.0, 1.5); //Left eye-ball
    drawSphere(0.2);
    popMatrix();

    popMatrix();
    pushMatrix();
    setDiffuseColor( 0.65, 0.65, 0.65 );
    setAmbientColor( 0.65, 0.65, 0.65 );
    translatef(-0.5, 0.0, -0.75);
    drawCylinder(l, 0.4, 0.4); //right eye

    pushMatrix();
    setDiffuseColor( 0.25, 0.65, 0.65 );
    setAmbientColor( 0.25, 0.65, 0.65 );
    translatef(0.0, 0.0, 1.5); //Right eye-ball
    drawSphere(0.2);
    popMatrix();

    popMatrix();
    popMatrix();
}

void upper_arm(float h) {					// draw the lower arm
    setDiffuseColor( 0.25, 0.25, 0.25 );
    setAmbientColor( 0.65, 0.85, 0.95 );
    y_box(h);
}

void lower_arm(float h) {					// draw the upper arm
    setDiffuseColor( 0.85, 0.75, 0.25 );
    setAmbientColor( 0.95, 0.75, 0.25 );
    pushMatrix();
    rotatef(-90.0, 1.0, 0.0, 0.0);
    drawCylinder( h, 0.15, 0.15 );
    popMatrix();
    /*
    pushMatrix();
    scalef( 1.0, 1.0, 0.7 );
    y_box(h);
    popMatrix();*/
}

void finger() { 			//	a finger
		glBegin(GL_QUADS);
		glVertex3f(0, 0, 0);
		glVertex3f(.7, 0, 0);
		glVertex3f(.7, 1, 0);
		glVertex3f(0, 1, 0);
		glEnd();
}

void hand(float angle, int side) {				//	draw the hand
		setDiffuseColor( 0.5, 0.5, 0.5 );
    setAmbientColor( 0.7, 0.7, 0.7 );
    
    angle = angle * 15;
    
    pushMatrix();	//	first
    scalef(0.3, 0.3, 0.3);
    scalef(1.0, 1.0, (side == RIGHT_HAND ? 1.0 : -1.0)); 
    translatef(-0.3, 0.0, 0.0);
    
    pushMatrix();	//	thumb side
    translatef(0.0, 0.0, 0.4);
    rotatef(30 - angle, 1.0, 0.0, 0.0);
    finger();
    translatef(0.0, 1.0, 0.0);
    
    rotatef(50, -1.0, 0.0, 0.0);
    finger();
    popMatrix();
    
    pushMatrix();	//	fingers side
    translatef(0.0, 0.0, -0.4);
    
    pushMatrix();	//	first finger
    translatef(0.4, 0.0, 0.0);	//	lower
    rotatef(60 - angle, -1.0, 0.0, 0.0);
		finger();

		translatef(0.0, 1.0, 0.0);  //	upper
		rotatef(90, 1.0, 0.0, 0.0);
		finger();
		popMatrix();
		
		pushMatrix();	//	second finger
		translatef(-0.4, 0.0, 0.0);	//	lower
		rotatef(60 - angle, -1.0, 0.0, 0.0);
		finger();
		translatef(0.0, 1.0, 0.0);  //	upper
		rotatef(90, 1.0, 0.0, 0.0); 
		finger();
		
		popMatrix();
		
		popMatrix();	//	end fingers side
		popMatrix();	//	last one	
}

void claw(float h) {
    setDiffuseColor( 0.25, 0.25, 0.85 );
    setAmbientColor( 0.25, 0.25, 0.85 );

    glBegin( GL_TRIANGLES );

    glNormal3d( 0.0, 0.0, 1.0);		// +z side
    glVertex3d( 0.5, 0.0, 0.5);
    glVertex3d(-0.5, 0.0, 0.5);
    glVertex3d( 0.5,   h, 0.5);

    glNormal3d( 0.0, 0.0, -1.0);	// -z side
    glVertex3d( 0.5, 0.0, -0.5);
    glVertex3d(-0.5, 0.0, -0.5);
    glVertex3d( 0.5,   h, -0.5);

    glEnd();

    glBegin( GL_QUADS );

    glNormal3d( 1.0,  0.0,  0.0);	// +x side
    glVertex3d( 0.5, 0.0,-0.5);
    glVertex3d( 0.5, 0.0, 0.5);
    glVertex3d( 0.5,   h, 0.5);
    glVertex3d( 0.5,   h,-0.5);

    glNormal3d( 0.0,-1.0, 0.0);		// -y side
    glVertex3d( 0.5, 0.0, 0.5);
    glVertex3d( 0.5, 0.0,-0.5);
    glVertex3d(-0.5, 0.0,-0.5);
    glVertex3d(-0.5, 0.0, 0.5);

    glEnd();
}

void y_box(float h) {

    glBegin( GL_QUADS );

    glNormal3d( 1.0 ,0.0, 0.0);			// +x side
    glVertex3d( 0.25,0.0, 0.25);
    glVertex3d( 0.25,0.0,-0.25);
    glVertex3d( 0.25,  h,-0.25);
    glVertex3d( 0.25,  h, 0.25);

    glNormal3d( 0.0 ,0.0, -1.0);		// -z side
    glVertex3d( 0.25,0.0,-0.25);
    glVertex3d(-0.25,0.0,-0.25);
    glVertex3d(-0.25,  h,-0.25);
    glVertex3d( 0.25,  h,-0.25);

    glNormal3d(-1.0, 0.0, 0.0);			// -x side
    glVertex3d(-0.25,0.0,-0.25);
    glVertex3d(-0.25,0.0, 0.25);
    glVertex3d(-0.25,  h, 0.25);
    glVertex3d(-0.25,  h,-0.25);

    glNormal3d( 0.0, 0.0, 1.0);			// +z side
    glVertex3d(-0.25,0.0, 0.25);
    glVertex3d( 0.25,0.0, 0.25);
    glVertex3d( 0.25,  h, 0.25);
    glVertex3d(-0.25,  h, 0.25);

    glNormal3d( 0.0, 1.0, 0.0);			// top (+y)
    glVertex3d( 0.25,  h, 0.25);
    glVertex3d( 0.25,  h,-0.25);
    glVertex3d(-0.25,  h,-0.25);
    glVertex3d(-0.25,  h, 0.25);

    glNormal3d( 0.0,-1.0, 0.0);			// bottom (-y)
    glVertex3d( 0.25,0.0, 0.25);
    glVertex3d(-0.25,0.0, 0.25);
    glVertex3d(-0.25,0.0,-0.25);
    glVertex3d( 0.25,0.0,-0.25);

    glEnd();
}

int main()
{
    // use classes and functions in VL lib 
    Mat4f a, b, c;
    a = vl_I;
    b = vl_0;
    b[0][2] = 3.0;
    c = a * b;
    //cout << c << endl; 

    ModelerControl controls[NUMCONTROLS ];

    controls[BASE_ROTATION] = ModelerControl("base rotation (theta)", -180.0, 180.0, 0.1, 0.0 );
    controls[DISPLACEMENT] = ModelerControl("base displacement (disp)", -180.0, 180.0, 0.1, 0.0 );
    controls[LOWER_TILT_LEFT] = ModelerControl("lower L arm tilt (phi_l)", 15.0, 95.0, 0.1, 55.0 );
    controls[LOWER_TILT_RIGHT] = ModelerControl("lower R arm tilt (phi_r)", 15.0, 95.0, 0.1, 55.0 );
    //controls[UPPER_TILT] = ModelerControl("upper arm tilt (psi)", 0.0, 135.0, 0.1, 30.0 );
    controls[CLAW_ROTATION] = ModelerControl("claw rotation (cr)", -30.0, 180.0, 0.1, 0.0 );
    controls[NECK_LENGTH] = ModelerControl("neck height (neckH)", 0.5, 10.0, 0.1, 0.8 );
    controls[UPPER_LENGTH_R] = ModelerControl("upper R arm length (h2_r)", 1, 10.0, 0.1, 1.5 );
    controls[LOWER_LENGTH_R] = ModelerControl("lower R arm  length (h3_r)", 1, 10.0, 0.1, 1.5 );
    controls[UPPER_LENGTH_L] = ModelerControl("upper L arm length (h2_l)", 1, 10.0, 0.1, 1.5 );
    controls[LOWER_LENGTH_L] = ModelerControl("lower L arm length (h3_l)", 1, 10.0, 0.1, 1.5 );
    controls[HAND_ANGLE_R] = ModelerControl("open/close R hand", 0, 1, 0.01, 0.2);
    controls[HAND_ANGLE_L] = ModelerControl("open/close L hand", 0, 1, 0.01, 0.2);
    controls[COMPOUND_R] = ModelerControl("rotate R arm and open hand", 0, 1, 0.01, 0.5);
    controls[COMPOUND_L] = ModelerControl("rotate L arm and open hand", 0, 1, 0.01, 0.5);
    controls[WALL1] = ModelerControl("wall 1 restitution coefficient", 0, 3, 0.5, 1);
    controls[WALL2] = ModelerControl("wall 2 restitution coefficient", 0, 3, 0.5, 1);
    controls[WALL3] = ModelerControl("wall 3 restitution coefficient", 0, 3, 0.5, 1);    


    // You should create a ParticleSystem object ps here and then
    // call ModelerApplication::Instance()->SetParticleSystem(ps)
    // to hook it up to the animator interface.
    
    ParticleSystem *ps = new ParticleSystem();
    // do some more particle system setup
    ModelerApplication::Instance()->SetParticleSystem(ps);

    ModelerApplication::Instance()->Init(&createRobotWallE, controls, NUMCONTROLS);

    return ModelerApplication::Instance()->Run();
}
